package;

import luxe.Entity;
import luxe.Sprite;
import luxe.Vector;
import luxe.Color;
import luxe.Particles;

import luxe.utils.Maths;
import luxe.tween.Actuate;

import luxe.components.sprite.SpriteAnimation;

import phoenix.Texture.FilterType;

/**
 * Person object, holds sprite and props
 * @author DevZero
 */
class Person extends Sprite {

	public var health 		: Float = 12;
	public var healthMax	: Float = 12;

	public var attackMin 		: Float = 5;
	public var attackMax 		: Float = 8;
	public var attackInterval	: Float = 1.6;
	public var attackRange 		: Float = 24;
	var _attackLast				: Float = 0;

	public var speed 		: Float = 30;
	public var scaleDefault : Vector = new Vector(0.87, 0.87);

	var anim 				: SpriteAnimation;

	var progress 				: Progress;

	// Instantiation
	public function new() {

		// trace("* new Person");

		// Give player a visual
		super({
			name: 'person',
			name_unique: true,
			pos: new Vector( 200, 200),
			size: new Vector(80, 52),
			texture: Luxe.resources.texture('assets/sprites/Person.png'),
			depth: 3
		});

		scale = scaleDefault.clone();

		initSprite();
		initProgress();

	}

	
	// Init
	override function init() {

		// trace('* Person Init');

		scale = new Vector(0.01, 0.01);
		Actuate.tween( scale, 0.6, {
			x: scaleDefault.x,
			y: scaleDefault.y,
		})
		.timescale(true)
		.ease( luxe.tween.easing.Back.easeOut );

	}


	public function moveTo( inEntity : Entity, removeOnComplete : Bool = false ) {

		if ( inEntity == null || inEntity.pos == null) { trace('* moveTo() Invalid input'); }

		var inPos : Vector = inEntity.pos.clone();
		var dist : Float = inPos.clone().subtract(pos).length;
		var tweenTime : Float = dist / speed;

		// trace('* Person tweenTime ${tweenTime}');

		// Flip facing direction based on movement direction
		flipx = (inPos.x > pos.x) ? true : false;

		Actuate.tween( pos, tweenTime, {
			x: inPos.x,
			y: inPos.y
		}, true)
		.timescale(true)
		.ease( luxe.tween.easing.Sine.easeInOut )
		.onComplete( function() {

			// FIXME: Workaround for tween continuing after destroy()
			if (health < 1) { return; }

			// If we arrived at a town, add to pop
			if (Std.is( inEntity, Town )) {
				var aTown : Town = cast inEntity;
				aTown.incrPopulation(1);
			}

			// Scale out and remove
			if (removeOnComplete) {
				kill();
			}
		});
	}


	public function playAttack() {
		// Main.playSound("hit");
		anim.animation = 'attack';
		anim.play();
	}


	override function update(dt:Float) {

		if (_attackLast > Luxe.time - attackInterval) {
			return;
		}

		// Update the attack time immediately so enemy doesn't hit at same time as player
		_attackLast = Luxe.time;

		// Within range?
		if ( Main.player.pos.clone().subtract(pos).length < attackRange) {
			Main.player.healthAdd( -1 * attackAmount() );
			
			playAttack();
			Luxe.camera.shake(2);
		}

	}



	public function healthAdd( i : Float = 1 ) : Float {
		
		health = Maths.clamp( health + i, 0, healthMax);

		if (health >= 0.5) {

			if (progress != null) {
				progress.setPercent( health / healthMax );
			}
		} else {
			kill();
		}
		
		return health;
		
	}

	public function attackAmount() : Float {
		return Math.round(Main.rng.float( attackMin, attackMax ));
	}


	public function kill() {

		Actuate.stop( this );

		trace('* Person kill() ${name} ${health}');

		if (health < 1) {
			Luxe.camera.shake(9);
			Main.playSound("coin");

			Main.world.incrScore( 5 );
		}

		// Remove from person array immediately
		Main.world.persons.remove( this );

		// Animate out
		Actuate.tween( scale, 0.7, {
			x: 0.01,
			y: 0.01,
		})
		.ease( luxe.tween.easing.Quart.easeIn )
		.onComplete( function() {
			destroy();
		});
	}

	
	override function ondestroy() {
		if (progress != null) { progress.destroy(); }

		super.ondestroy();
	}


	function initProgress() {
		progress = new Progress(new Color().rgb(0xFA5858));
		progress.pos.add_xyz( size.x / 2 + 4, -4);
		progress.parent = this;
	}


	function initSprite() {

		anim = new SpriteAnimation({ name: 'personanim' });
		add(anim);

		
		var animation_json = '
			{
				"idle" : {
					"frame_size":{ "x":"80", "y":"52" },
					"frameset": ["1-6"],
					"events" : [],
					"pingpong": "true",
					"loop": "true",
					"speed": "12"
				},
				"attack" : {
					"frame_size":{ "x":"80", "y":"52" },
					"frameset": ["7-12"],
					"events" : [{"frame": 12, "event": "attack.done"}],
					"pingpong": "false",
					"loop": "false",
					"speed": "16"
				},
				"walk_glitch" : {
					"frame_sources":[{
						"frame":8, "x":0, "y":0, "w":96, "h":96
					}],
					"frame_size":{ "x":"48", "y":"48" },
					"frameset": ["1-8","9","10","hold 10","11 hold 5", "12"],
					"pingpong":"false",
					"loop": "true",
					"speed": "18"
				}
			}
		';

		anim.add_from_json( animation_json );

		anim.animation = 'idle';
		anim.play();

		// Do thing on sprite anim events
		events.listen('attack.done', function(e) {
			// trace('* P - Attack anim done!');
			anim.animation = 'idle';
			anim.play();
		});


	}

}