package;

import luxe.Entity;
import luxe.Sprite;
import luxe.Vector;
import luxe.Color;
import luxe.Particles;
import luxe.Game;
import luxe.Input;

import luxe.utils.Maths;
import luxe.tween.Actuate;

import luxe.components.sprite.SpriteAnimation;

import phoenix.Texture.FilterType;

/**
 * Player object, holds sprite and props
 * @author DevZero
 */
class Player extends Sprite {

	public var health 			: Float = 50;
	public var healthMax 		: Float = 50;

	public var attackMin 		: Float = 5;
	public var attackMax 		: Float = 8;
	public var attackInterval	: Float = 1.2;
	public var attackRange 		: Float = 30;
	var _attackLast				: Float = 0;

	public var speed 			: Float = 50;
	var anim 					: SpriteAnimation;


	var progress 				: Progress;

	// Instantiation
	public function new() {

		trace("* new Player");

		// Give player a visual
		super({
			name: 'player',
			pos: new Vector(0,0),
			size: new Vector(66, 49),
			centered: true,
			texture: Luxe.resources.texture('assets/sprites/Character.png'),
			depth: 5
		});

		// texture.filter_min = texture.filter_mag = FilterType.nearest;

		// size.multiplyScalar(2);

		pos = Luxe.screen.mid;
		initSprite();
		initProgress();

	}

	
	// Init
	override function init() {

		trace('* Player Init');
		
	}


	public function moveTo( inPos : Vector, removeOnComplete : Bool = false  ) {

		var dist : Float = inPos.clone().subtract(pos).length;
		var tweenTime : Float = dist / speed;

		// trace('* Skel tweenTime ${tweenTime}');

		// Flip facing direction based on movement direction
		flipx = (inPos.x > pos.x) ? true : false;

		Actuate.tween( pos, tweenTime, {
			x: inPos.x,
			y: inPos.y
		})
		.timescale(true)
		.ease( luxe.tween.easing.Cubic.easeOut )
		.onComplete( function() {

			// Scale out and remove
			if (removeOnComplete) {
				Actuate.tween( scale, 0.7, {
					x: 0.01,
					y: 0.01,
				})
				.timescale(true)
				.ease( luxe.tween.easing.Quart.easeIn )
				.onComplete( function() {
					destroy();
				});
			}
		});
	}


	public function playAttack() {
		Main.playSound("hit");
		anim.animation = 'attack';
		anim.play();
	}


	override function onmousedown( e:MouseEvent ) {

		Main.playClick();
		moveTo( e.pos );

	}


	override function update(dt:Float) {

		if (_attackLast > Luxe.time - attackInterval) {
			return;
		}

		// Search for closest
		var closest : Person = Main.world.findClosestPerson( pos );
		if (closest == null) { return; }

		// Within range
		if ( closest.pos.clone().subtract(pos).length < attackRange) {
			closest.healthAdd( -1 * attackAmount() );
			
			playAttack();
			Luxe.camera.shake(6);

			_attackLast = Luxe.time;
		}


	}


	public function healthAdd( i : Float = 1 ) : Float {
		
		health = Maths.clamp( health + i, 0, healthMax);

		if (health >= 0.5) {

			updateProgress();
		} else {
			respawn();
		}
		
		return health;
		
	}


	public function respawn() {

		var newPos : Vector = Luxe.screen.mid.clone();
		if (Main.world.outposts.length > 0) {
			newPos = Main.world.outposts[0].pos;
		}

		pos = newPos.clone();
		health = healthMax;

		updateProgress();

		Main.playSound("throw");

	}


	function updateProgress() {
		if (progress != null) {
			progress.setPercent( health / healthMax );
		}
	}


	public function attackAmount() : Float {
		return Math.round(Main.rng.float( attackMin, attackMax ));
	}

	
	override function ondestroy() {
		if (progress != null) { progress.destroy(); }

		super.ondestroy();
	}


	function initProgress() {
		progress = new Progress(new Color().rgb(0x00C7DF));
		progress.pos.add_xyz( size.x / 2 + 4, -4);
		progress.parent = this;
	}


	function initSprite() {

		anim = new SpriteAnimation({ name: 'playeranim' });
		add(anim);

		
		var animation_json = '
			{
				"idle" : {
					"frame_size":{ "x":"66", "y":"49" },
					"frameset": ["1-6"],
					"events" : [],
					"pingpong": "true",
					"loop": "true",
					"speed": "12"
				},
				"attack" : {
					"frame_size":{ "x":"66", "y":"49" },
					"frameset": ["7-12"],
					"events" : [{"frame": 12, "event": "attack.done"}],
					"pingpong": "false",
					"loop": "false",
					"speed": "16"
				},
				"walk_glitch" : {
					"frame_sources":[{
						"frame":8, "x":0, "y":0, "w":96, "h":96
					}],
					"frame_size":{ "x":"48", "y":"48" },
					"frameset": ["1-8","9","10","hold 10","11 hold 5", "12"],
					"pingpong":"false",
					"loop": "true",
					"speed": "18"
				}
			}
		';

		anim.add_from_json( animation_json );

		anim.animation = 'idle';
		anim.play();

		// Do thing on sprite anim events
		events.listen('attack.done', function(e) {
			// trace('* P - Attack anim done!');
			anim.animation = 'idle';
			anim.play();
		});


	}

}